import { bttv_get_url } from "./bttv_api";
import { map_parallel } from "./helpers";

const emote_regex = /bttv:([^\s]+)/gi;

async function process_text(text: string): Promise<string> {
  const matches = [...text.matchAll(emote_regex)];
  const codes = new Set(matches.map(([_, code]) => code));
  console.log("[lmlfc-bttv]", `searching ${[...codes].length} emotes`);
  const emotes = await map_parallel(codes, bttv_get_url);
  console.debug("matches", matches, "codes", codes, "emotes", emotes);

  for (const emote of matches.reverse()) {
    const [match, code] = emote;
    console.debug("match", match, "code", code);

    if (emote.index == undefined) continue;
    const index = emote.index;

    if (!emotes.has(code)) continue;
    const url = emotes.get(code);
    console.debug("index", index, "url", url);

    text =
      text.substring(0, index) +
      `[not]${url}[/not]` +
      text.substring(index + match.length);
  }

  return text.replace(/\[\/not\]\s+\[not\]/g, "[/not][not]");
}

(() => {
  const cb_form = document.querySelector("#mgc_cb_evo_form");
  if (!(cb_form instanceof HTMLFormElement)) return;

  const cb_send = document.querySelector(
    "#mgc_cb_evo_form > input[type=image]:nth-child(2)",
  );
  if (!(cb_send instanceof HTMLInputElement)) return;

  const cb_input = document.querySelector("#mgc_cb_evo_input");
  if (!(cb_input instanceof HTMLInputElement)) return;

  const bttv_btn = (() => {
    const btn = document.createElement("a");
    btn.style.setProperty("cursor", "pointer");
    btn.style.setProperty("margin-right", "4px");

    const img = document.createElement("img");
    img.setAttribute("src", chrome.runtime.getURL("img/sb_button.png"));
    img.setAttribute("alt", "bttv emotes ersetzen und absenden");
    img.setAttribute("title", "bttv:* emotes ersetzen und absenden");
    img.style.setProperty("vertical-align", "middle");

    btn.addEventListener("click", async () => {
      cb_input.value = await process_text(cb_input.value);
      cb_input.focus();
      cb_send.click();
    });

    btn.appendChild(img);
    return btn;
  })();

  cb_form.insertBefore(bttv_btn, cb_send);

  console.log("[lmlfc-bttv]", `loaded`);
})();
